@@ -76,6 +76,7 @@ class MindsDBLexer(Lexer):
         AND, OR, NOT, IS, IS_NOT, TYPECAST,
         IN, NOT_IN, LIKE, NOT_LIKE, CONCAT, BETWEEN, WINDOW, OVER, PARTITION_BY,
         VECT_L2, VECT_L1, VECT_INNER, VECT_COS, VECT_HAMM, VECT_JACC,
+        ARR_CONTAINS, ARR_CONTAINED, ARR_OVERLAP,
         JSON_GET, JSON_GET_STR, INTERVAL,
 
         # Data types
@@ -126,8 +127,8 @@ class MindsDBLexer(Lexer):
     ANOMALY = r'\bANOMALY\b'
     DETECTION = r'\bDETECTION\b'
 
-    KNOWLEDGE_BASE = r'\bKNOWLEDGE[_|\s]BASE\b'
-    KNOWLEDGE_BASES = r'\bKNOWLEDGE[_|\s]BASES\b'
+    KNOWLEDGE_BASE = r'\bKNOWLEDGE([_]|[\s]+)BASE\b'
+    KNOWLEDGE_BASES = r'\bKNOWLEDGE([_]|[\s]+)BASES\b'
     SKILL = r'\bSKILL\b'
     AGENT = r'\bAGENT\b'
 
@@ -167,7 +168,7 @@ class MindsDBLexer(Lexer):
     STATUS = r'\bSTATUS\b'
     GLOBAL = r'\bGLOBAL\b'
     PROCEDURE = r'\bPROCEDURE\b'
-    PRIMARY_KEY = r'\bPRIMARY[\s]KEY\b'
+    PRIMARY_KEY = r'\bPRIMARY[\s]+KEY\b'
     DEFAULT = r'\bDEFAULT\b'
     FUNCTION = r'\bFUNCTION\b'
     INDEX = r'\bINDEX\b'
@@ -214,8 +215,8 @@ class MindsDBLexer(Lexer):
     ON = r'\bON\b'
     ASC = r'\bASC\b'
     DESC = r'\bDESC\b'
-    NULLS_FIRST = r'\bNULLS FIRST\b'
-    NULLS_LAST = r'\bNULLS LAST\b'
+    NULLS_FIRST = r'\bNULLS[\s]+FIRST\b'
+    NULLS_LAST = r'\bNULLS[\s]+LAST\b'
     WITH = r'\bWITH\b'
     SELECT = r'\bSELECT\b'
     DISTINCT = r'\bDISTINCT\b'
@@ -224,15 +225,15 @@ class MindsDBLexer(Lexer):
     WHERE = r'\bWHERE\b'
     LIMIT = r'\bLIMIT\b'
     OFFSET = r'\bOFFSET\b'
-    GROUP_BY = r'\bGROUP BY\b'
+    GROUP_BY = r'\bGROUP[\s]+BY\b'
     HAVING = r'\bHAVING\b'
-    ORDER_BY = r'\bORDER BY\b'
+    ORDER_BY = r'\bORDER[\s]+BY\b'
     STAR = r'\*'
     FOR = r'\bFOR\b'
     UPDATE = r'\bUPDATE\b'
     # FETCH FIRST and FETCH NEXT are the same
-    FETCH_FIRST = r'\bFETCH[\s](FIRST|NEXT)\b'
-    ROWS_ONLY = r'\bROWS[\s]ONLY\b'
+    FETCH_FIRST = r'\bFETCH[\s]+(FIRST|NEXT)\b'
+    ROWS_ONLY = r'\bROWS[\s]+ONLY\b'
     ROLLUP = r'\bROLLUP\b'
 
     JOIN = r'\bJOIN\b'
@@ -291,6 +292,10 @@ class MindsDBLexer(Lexer):
     VECT_HAMM = r'<~>'
     VECT_JACC = r'<%>'
 
+    ARR_CONTAINS = r'@>'
+    ARR_CONTAINED = r'<@'
+    ARR_OVERLAP = r'&&'
+
     JSON_GET_STR = r'->>'
     JSON_GET = r'->'
     PLUS = r'\+'
@@ -325,7 +330,7 @@ class MindsDBLexer(Lexer):
     INTERVAL = r'\bINTERVAL\b'
     WINDOW = r'\bWINDOW\b'
     OVER = r'\bOVER\b'
-    PARTITION_BY = r'\bPARTITION BY\b'
+    PARTITION_BY = r'\bPARTITION[\s]+BY\b'
 
     # Data types
     NULL = r'\bNULL\b'
@@ -346,12 +351,10 @@ class MindsDBLexer(Lexer):
 
     @_(r"'(?:\\.|[^'])*(?:''(?:\\.|[^'])*)*'")
     def QUOTE_STRING(self, t):
-        t.value = t.value.replace('\\"', '"').replace("\\'", "'").replace("''", "'")
         return t
 
-    @_(r'"(?:\\.|[^"])*"')
+    @_(r'"(?:\\.|[^"])*(?:""(?:\\.|[^"])*)*"')
     def DQUOTE_STRING(self, t):
-        t.value = t.value.replace('\\"', '"').replace("\\'", "'")
         return t
 
     @_(r'\n+')
