@@ -1,3 +1,5 @@
+import re
+
 from sly import Parser
 from mindsdb_sql_parser.ast import *
 from mindsdb_sql_parser.ast.drop import DropDatabase, DropView
@@ -19,12 +21,12 @@ from mindsdb_sql_parser.ast.mindsdb.trigger import CreateTrigger, DropTrigger
 from mindsdb_sql_parser.ast.mindsdb.latest import Latest
 from mindsdb_sql_parser.ast.mindsdb.evaluate import Evaluate
 from mindsdb_sql_parser.ast.mindsdb.knowledge_base import CreateKnowledgeBase, DropKnowledgeBase, \
-    CreateKnowledgeBaseIndex, DropKnowledgeBaseIndex, EvaluateKnowledgeBase
+   AlterKnowledgeBase, CreateKnowledgeBaseIndex, DropKnowledgeBaseIndex, EvaluateKnowledgeBase
 from mindsdb_sql_parser.ast.mindsdb.skills import CreateSkill, DropSkill, UpdateSkill
 from mindsdb_sql_parser.exceptions import ParsingException
 from mindsdb_sql_parser.ast.mindsdb.retrain_predictor import RetrainPredictor
 from mindsdb_sql_parser.ast.mindsdb.finetune_predictor import FinetunePredictor
-from mindsdb_sql_parser.utils import ensure_select_keyword_order, JoinType, tokens_to_string
+from mindsdb_sql_parser.utils import ensure_select_keyword_order, JoinType, tokens_to_string, unquote
 from mindsdb_sql_parser.logger import ParserLogger
 
 from mindsdb_sql_parser.lexer import MindsDBLexer
@@ -93,6 +95,7 @@ class MindsDBParser(Parser):
        'create_trigger',
        'drop_trigger',
        'create_kb',
+       'alter_kb',
        'drop_kb',
        'evaluate_kb',
        'create_skill',
@@ -111,9 +114,6 @@ class MindsDBParser(Parser):
     @_(
         'CREATE KNOWLEDGE_BASE if_not_exists_or_empty identifier USING kw_parameter_list',
         'CREATE KNOWLEDGE_BASE if_not_exists_or_empty identifier',
-        # from select
-        'CREATE KNOWLEDGE_BASE if_not_exists_or_empty identifier FROM LPAREN select RPAREN USING kw_parameter_list',
-        'CREATE KNOWLEDGE_BASE if_not_exists_or_empty identifier FROM LPAREN select RPAREN',
     )
     def create_kb(self, p):
         params = getattr(p, 'kw_parameter_list', {})
@@ -124,7 +124,7 @@ class MindsDBParser(Parser):
         model = params.pop('model', None)
         storage = params.pop('storage', None)
 
-        if isinstance(model, str):
+        if isinstance(storage, str):
             # convert to identifier
             storage = Identifier(storage)
 
@@ -143,6 +143,19 @@ class MindsDBParser(Parser):
             if_not_exists=if_not_exists
         )
 
+    @_(
+        'ALTER KNOWLEDGE_BASE identifier USING kw_parameter_list',
+    )
+    def alter_kb(self, p):
+        params = getattr(p, 'kw_parameter_list', {})
+        name = p.identifier
+        params = {k.lower(): v for k, v in params.items()}  # case insensitive
+
+        return AlterKnowledgeBase(
+            name=name,
+            params=params
+        )
+
     @_('CREATE INDEX ON KNOWLEDGE_BASE identifier')
     def create_index(self, p):
         return CreateKnowledgeBaseIndex(name=p.identifier)
@@ -165,7 +178,7 @@ class MindsDBParser(Parser):
     # -- Skills --
     @_('CREATE SKILL if_not_exists_or_empty identifier USING kw_parameter_list')
     def create_skill(self, p):
-        params = p.kw_parameter_list
+        params = {k.lower(): v for k, v in p.kw_parameter_list.items()}
 
         return CreateSkill(
             name=p.identifier,
@@ -180,12 +193,12 @@ class MindsDBParser(Parser):
 
     @_('UPDATE SKILL identifier SET kw_parameter_list')
     def update_skill(self, p):
-        return UpdateSkill(name=p.identifier, updated_params=p.kw_parameter_list)
+        return UpdateSkill(name=p.identifier, updated_params={k.lower(): v for k, v in p.kw_parameter_list.items()})
 
     # -- Agent --
     @_('CREATE AGENT if_not_exists_or_empty identifier USING kw_parameter_list')
     def create_agent(self, p):
-        params = p.kw_parameter_list
+        params = {k.lower(): v for k, v in p.kw_parameter_list.items()}
 
         return CreateAgent(
             name=p.identifier,
@@ -201,12 +214,12 @@ class MindsDBParser(Parser):
     @_('UPDATE AGENT identifier SET kw_parameter_list')
     @_('ALTER AGENT identifier USING kw_parameter_list')
     def update_agent(self, p):
-        return UpdateAgent(name=p.identifier, updated_params=p.kw_parameter_list)
+        return UpdateAgent(name=p.identifier, updated_params={k.lower(): v for k, v in p.kw_parameter_list.items()})
 
     # -- ChatBot --
     @_('CREATE CHATBOT identifier USING kw_parameter_list')
     def create_chat_bot(self, p):
-        params = p.kw_parameter_list
+        params = {k.lower(): v for k, v in p.kw_parameter_list.items()}
 
         database = Identifier(params.pop('database'))
         model_param = params.pop('model', None)
@@ -225,7 +238,7 @@ class MindsDBParser(Parser):
 
     @_('UPDATE CHATBOT identifier SET kw_parameter_list')
     def update_chat_bot(self, p):
-        return UpdateChatBot(name=p.identifier, updated_params=p.kw_parameter_list)
+        return UpdateChatBot(name=p.identifier, updated_params={k.lower(): v for k, v in p.kw_parameter_list.items()})
 
     @_('DROP CHATBOT identifier')
     def drop_chat_bot(self, p):
@@ -559,7 +572,8 @@ class MindsDBParser(Parser):
     def show(self, p):
         modes = getattr(p, 'show_modifier_list', None)
         return Show(
-            category=p.show_category,
+            # replace multiple spaces
+            category=re.sub(r"[\s]+", " ", p.show_category),
             modes=modes
         )
 
@@ -1106,14 +1120,15 @@ class MindsDBParser(Parser):
         select.cte = p.ctes
         return select
 
-    @_('ctes COMMA identifier cte_columns_or_nothing AS LPAREN select RPAREN')
+    @_('ctes COMMA identifier cte_columns_or_nothing AS LPAREN select RPAREN',
+       'ctes COMMA identifier cte_columns_or_nothing AS LPAREN union RPAREN')
     def ctes(self, p):
         ctes = p.ctes
         ctes = ctes + [
             CommonTableExpression(
                 name=p.identifier,
                 columns=p.cte_columns_or_nothing,
-                query=p.select)
+                query=p[6])
         ]
         return ctes
 
@@ -1194,12 +1209,12 @@ class MindsDBParser(Parser):
 
     @_('ordering_term NULLS_FIRST')
     def ordering_term(self, p):
-        p.ordering_term.nulls = p.NULLS_FIRST
+        p.ordering_term.nulls = 'NULLS FIRST'
         return p.ordering_term
 
     @_('ordering_term NULLS_LAST')
     def ordering_term(self, p):
-        p.ordering_term.nulls = p.NULLS_LAST
+        p.ordering_term.nulls = 'NULLS LAST'
         return p.ordering_term
 
     @_('ordering_term DESC')
@@ -1663,6 +1678,9 @@ class MindsDBParser(Parser):
        'expr JSON_GET constant',
        'expr JSON_GET_STR constant',
        'expr NOT_IN expr',
+       'expr ARR_CONTAINS expr',
+       'expr ARR_CONTAINED expr',
+       'expr ARR_OVERLAP expr',
        'expr IN expr',)
     def expr(self, p):
         if p[1] == ':=':
@@ -2006,11 +2024,11 @@ class MindsDBParser(Parser):
 
     @_('QUOTE_STRING')
     def quote_string(self, p):
-        return p[0].strip('\'')
+        return unquote(p[0]).strip('\'')
 
     @_('DQUOTE_STRING')
     def dquote_string(self, p):
-        return p[0].strip('\"')
+        return unquote(p[0], is_double_quoted=True).strip('\"')
 
     # for raw query
 
